@@ -1,8 +1,11 @@
 import email
+from email.utils import formatdate
 import rfc822
 import urllib
 import urlparse
 
+from django.conf import settings
+from django.core.mail.message import make_msgid
 from django.db import models
 from django_mailbox.transports import Pop3Transport, ImapTransport,\
         MaildirTransport, MboxTransport, BabylTransport, MHTransport, \
@@ -33,7 +36,32 @@ class Mailbox(models.Model):
             null=True,
             default=None,
             )
+    from_email = models.CharField(
+            max_length=255,
+            help_text="""
+                Example: MailBot &lt;mailbot@yourdomain.com&gt;
+                <br />
+                'From' header to set for outgoing email.
+                <br />
+                <br />
+                If you do not use this e-mail inbox for outgoing mail, this 
+                setting is unnecessary.
+                <br />
+                If you send e-mail without setting this, your 'From' header will
+                be set to match the setting `DEFAULT_FROM_EMAIL`.
+            """,
+            blank=True,
+            null=True,
+            default=None,
+            )
     active = models.BooleanField(
+            help_text="""
+                Check this e-mail inbox for new e-mail messages during polling
+                cycles.  This checkbox does not have an effect upon whether
+                mail is collected here when this mailbox receives mail from a
+                pipe, and does not affect whether e-mail messages can be
+                dispatched from this mailbox.
+            """,
             blank=True,
             default=True,
             )
@@ -106,11 +134,25 @@ class Mailbox(models.Model):
         return conn
 
     def process_incoming_message(self, message):
+        msg = self._process_message(message)
+        msg.outgoing = False
+        msg.save()
+        message_received.send(sender=self, message=msg)
+        return msg
+
+    def record_outgoing_message(self, message):
+        msg = self._process_message(message)
+        msg.outgoing = True
+        msg.save()
+        return msg
+
+    def _process_message(self, message):
         msg = Message()
         msg.mailbox = self
         msg.subject = message['subject'][0:255]
         msg.message_id = message['message-id'][0:255]
-        msg.address = rfc822.parseaddr(message['from'])[1][0:255]
+        msg.from_header = message['from']
+        msg.to_header = message['to']
         msg.body = message.as_string()
         if message['in-reply-to']:
             try:
@@ -118,16 +160,6 @@ class Mailbox(models.Model):
             except IndexError:
                 pass
         msg.save()
-        if message['references']:
-            references = message['references'].split('\n')
-            for reference in references:
-                try:
-                    msg.references.add(
-                        Message.objects.filter(message_id=reference.strip())[0]
-                    )
-                except IndexError:
-                    pass
-        message_received.send(sender=self, message=msg)
         return msg
 
     def get_new_mail(self):
@@ -168,13 +200,10 @@ class Message(models.Model):
         blank=True,
         null=True,
     )
-    references = models.ManyToManyField(
-        'django_mailbox.Message', 
-        related_name='referenced_by',
-        blank=True,
-        null=True,
+    from_header = models.CharField(
+        max_length=255,
     )
-    address = models.CharField(max_length=255)
+    to_header = models.TextField()
     outgoing = models.BooleanField(
         default=False,
         blank=True,
@@ -191,12 +220,58 @@ class Message(models.Model):
     outgoing_messages = OutgoingMessageManager()
 
     @property
-    def from_address(self):
-        return self.get_email_object(self)['from']
+    def address(self):
+        """Property allowing one to get the relevant address(es).
+        
+        In earlier versions of this library, the model had an `address` field
+        storing the e-mail address from which a message was received.  During
+        later refactorings, it became clear that perhaps storing sent messages
+        would also be useful, so the address field was replaced with two
+        separate fields.
+        
+        """
+        if self.outgoing:
+            return self.to_addresses()
+        else:
+            return self.from_addresses()
 
     @property
-    def to_address(self):
-        return self.get_email_object(self)['to']
+    def from_address(self):
+        return rfc822.parseaddr(self.from_header)[1]
+
+    @property
+    def to_addresses(self):
+        addresses = []
+        for address in self.to_header.split(','):
+            addresses.append(
+                    rfc822.parseaddr(
+                            address
+                        )[1]
+                )
+        return addresses
+
+    def reply(self, message):
+        """Sends a message as a reply to this message instance.
+        
+        Although Django's e-mail processing will set both Message-ID
+        and Date upon generating the e-mail message, we will not be able
+        to retrieve that information through normal channels, so we must
+        pre-set it.
+
+        """
+        if self.mailbox.from_email:
+            message.from_email = self.mailbox.from_email
+        else:
+            message.from_email = settings.DEFAULT_FROM_EMAIL
+        message.extra_headers['Message-ID'] = make_msgid()
+        message.extra_headers['Date'] = formatdate()
+        message.extra_headers['In-Reply-To'] = self.message_id
+        message.send()
+        return self.mailbox.record_outgoing_message(
+                email.message_from_string(
+                    message.message().as_string()
+                )
+            )
 
     def get_email_object(self):
         return email.message_from_string(self.body)
